<?php

namespace App\Controller;

use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;
use LogicException;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Twig\Environment;

abstract class BaseController extends AbstractController
{
    protected const EDIT = "edit";
    protected const NEW = "new";

    public function isSlide(): string
    {
        return $this->getRequest()->isXmlHttpRequest() ? "_slide.html.twig" : ".html.twig";
    }

    protected function blankSlide(string $html = ""): Response
    {
        return $this->render("default/slide/blank.html.twig", [
            'html' => $html,
        ]);
    }

    public function renderBlock(string $template, string $block, array $context = [])
    {
        $twig = $this->getTwig();
        $template = $twig->load($template);

        return new Response($template->renderBlock($block, $context));
    }

    protected function getTwig(): Environment
    {
        return $this->container->get("twig");
    }

    protected function widget(string $template, array $context = []): JsonResponse
    {
        return $this->json([
            'status' => "OK",
            'html' => $this->renderView($template, $context),
        ]);
    }

    protected function baseIndex(
        $datatable,
        $newUri = null,
        $title = null,
        $context = [],
        $viewTemplate = 'default/index.html.twig'
    ): Response
    {
//        if ($this->isAjax()) {
//            return $this->render($grid);
//        }

        return $this->renderAjax(
            'default/slide/index.html.twig',
            $viewTemplate,
            array_merge([
                'new_uri' => $newUri,
                'datatable' => $datatable,
                'title' => $title,
            ], $context)
        );
    }

    /**
     * @throws NotFoundExceptionInterface
     * @throws ContainerExceptionInterface
     */
    protected function baseDelete(
        mixed                  $entity,
        string                 $message,
        EntityManagerInterface $entityManager = null,
        callable               $preDelete = null
    ): Response
    {
        $em = $entityManager ?? $this->getEm();

        $request = $this->getRequest();
        if ($request->isMethod(Request::METHOD_POST)) {
            if (null !== $preDelete) {
                $preDelete();
            }

            $em->remove($entity);
            $em->flush();

            return $this->renderAjax(
                'default/slide/success.html.twig',
                'default/delete.html.twig',
                [
                    'message' => 'successfully deleted',
                ]
            );
        } else {
            return $this->renderAjax(
                'default/slide/delete.html.twig',
                'default/delete.html.twig',
                [
                    'message' => $message,
                ]
            );
        }
    }

    public function form(FormInterface $form, string|array $title, string|null $template = "default/slide/form.html.twig"): Response
    {
        if (is_array($title)) {
            $title = implode(" ", $title);
        }

        $context = [
            'form' => $form->createView(),
            "title" => $title
        ];

        return $this->render($template, $context);
    }

    protected function successSlideOrRedirect(
        $redirectRoute,
        $routeParams = [],
        $message = null
    ): RedirectResponse|Response
    {
        if ($this->isAjax()) {
            return $this->successSlide($message);
        } else {
            return $this->redirectToRoute($redirectRoute, $routeParams);
        }
    }

    protected function successSlide($message = null, WidgetReloader|null $widgetReloader = null): Response
    {
        if ($widgetReloader) {
            foreach ($widgetReloader->getWidgets() as $widget) {
                $this->renderWidget($widget);
            }
        }

        return $this->render('default/slide/success.html.twig', [
            'message' => $message,
            'widgetReloader' => $widgetReloader
        ]);
    }


    protected function closeSlide(WidgetReloader|null $widgetReloader = null): Response
    {
        if (!$widgetReloader) {
            $widgetReloader = new WidgetReloader();
        }
        foreach ($widgetReloader->getWidgets() as $widget) {
            $this->renderWidget($widget);
        }

        $context = ["widgetReloader" => $widgetReloader];

        return $this->render("default/slide/close.html.twig", $context);
    }

    protected function errorSlide($message = null): Response
    {
        return $this->render('default/slide/error.html.twig', ['message' => $message]);
    }

    protected function renderAjax($ajaxTemplate, $pageTemplate, $context = []): Response
    {
        $request = $this->getRequest();
        if ($request->isXmlHttpRequest()) {
            return $this->render($ajaxTemplate, $context);
        } else {
            return $this->render($pageTemplate, $context);
        }
    }

    protected function renderEdit($form): Response
    {
        if ($form instanceof FormInterface) {
            $form = $form->createView();
        }

        return $this->renderAjax(
            'default/slide/edit.html.twig',
            'default/edit.html.twig',
            [
                'form' => $form,
            ]
        );
    }

    protected function renderNew(
        $form,
        $title = null,
        $ajaxTemplate = 'default/slide/new.html.twig',
        $pageTemplate = 'default/new.html.twig'
    ): Response
    {
        if ($form instanceof FormInterface) {
            $form = $form->createView();
        }

        return $this->renderAjax(
            $ajaxTemplate,
            $pageTemplate,
            [
                'form' => $form,
                'title' => $title,
            ]
        );
    }

    protected function renderShow($showTemplate, $context = [])
    {

        $parent = $this->isAjax() ? 'default/slide/show.html.twig' : 'base.html.twig';


        return $this->render($showTemplate,
            array_merge($context, [
                'parent' => $parent,
                'isSlide' => $this->isAjax()
            ])
        );
    }

    protected function isAjax(): bool
    {
        return $this->getRequest()->isXmlHttpRequest();
    }

    protected function getRequest(): Request|bool|null
    {
        try {
            return $this->container->get('request_stack')->getCurrentRequest();
        } catch (NotFoundExceptionInterface|ContainerExceptionInterface $e) {
            return false;
        }
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    protected function getEm(): ?EntityManagerInterface
    {
        return $this->container->get('em');
    }

    public static function getSubscribedServices(): array
    {
        return array_merge(parent::getSubscribedServices(), [
            'em' => '?' . EntityManagerInterface::class,
        ]);
    }

    public function slide(
        FormInterface $form,
        string        $slide,
        string        $class = "",
        array         $context = [],
        string|null   $template = null
    ): Response
    {
        $context = array_merge($context, [
            'form' => $form->createView(),
            "class" => $class,
        ]);

        return $this->render($template ?: "default/slide/$slide.html.twig", $context);
    }

    protected function handleWidget(string|array $parameter): bool|JsonResponse
    {
        $request = $this->getRequest();

        if ($widget = $request->get("widget")) {
            if (is_array($parameter)) {
                $context = $parameter;
                $parameter = $parameter["parameter"];
            } else {
                $context = [$parameter => $request->attributes->get($parameter)];
            }

            $context = array_merge($context, $request->query->all());

            return $this->widget("{$parameter}/widgets/{$widget}.html.twig", $context);
        } else {
            return false;
        }
    }

    public function slideOrRender(string $regular, string $slide, array $context = []): Response
    {
        if ($this->getRequest()->isXmlHttpRequest()) {
            return $this->render($slide, $context);
        }

        return $this->render($regular, $context);
    }

    public function disableFilter(string $filter): void
    {
        if (!property_exists($this, "entityManager")) {
            throw new LogicException("Property \"entityManager\" does not exist in " . get_class($this));
        }

        $filters = $this->entityManager->getFilters();
        $filters->disable($filter);
    }

    public function enableFilter(string $filter): void
    {
        if (!property_exists($this, "entityManager")) {
            throw new LogicException("Property \"entityManager\" does not exist in " . get_class($this));
        }

        $filters = $this->entityManager->getFilters();
        $filters->enable($filter);
    }

    protected function repo(string $class): EntityRepository
    {
        if (!property_exists($this, "entityManager")) {
            $this->entityManager = $this->getEm();
        }

        return $this->entityManager->getRepository($class);
    }

    public function renderWidget(Widget $widget): Widget
    {
        $widget->setHtml($this->renderView($widget->getWidget(), $widget->getContext()));
        return $widget;
    }
}