<?php

namespace App\Controller;

use Doctrine\ORM\EntityManagerInterface;
use Fourbis\CmsBundle\Entity\Post\Post;
use Fourbis\CmsBundle\Entity\Post\PostTranslation;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Serializer\SerializerInterface;

#[Route("/{_locale}/rss")]
class FeedController extends BaseController
{
    private EntityManagerInterface $entityManager;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    #[Route("/get", name: "get_rss", options: ['expose' => true])]
    public function index(RequestStack $requestStack): Response
    {
        $request = $requestStack->getCurrentRequest();
        $baseUrl = $request->getSchemeAndHttpHost();
        $locale = $request->getLocale();
        $posts = $this->getAdvertisementNewsUpdates($baseUrl, $locale);

        if (empty($posts)) {
            return new JsonResponse(['error' => 'No posts found'], Response::HTTP_NOT_FOUND);
        }

        $rssFeed = $this->generateRssFeed($posts, $baseUrl, $locale, $request);
        $response = new Response($rssFeed->asXML());
        $response->headers->set('Content-Type', 'text/xml');
        return $response;
    }

    private function getAdvertisementNewsUpdates(string $baseUrl, string $locale): array
    {
        return $this->entityManager->getRepository(Post::class)->createQueryBuilder('p')
            ->select('p.uuid AS post_uuid', 'pt.uuid AS translation_uuid', 'pt.title', 'pt.description', 'p.publishedAt', 'pt.content', 'pt.url', "CONCAT(:baseUrl, '/', :locale, pt.fullUrl) AS absoluteUrl, pt.locale")
            ->leftJoin('p.translations', 'pt')
            ->where('pt.fullUrl LIKE :url')
            ->andWhere('pt.locale = :locale')
            ->setParameter('url', '%/blog/%')
            ->setParameter('baseUrl', $baseUrl)
            ->setParameter('locale', $locale)
            ->orderBy('p.publishedAt', 'DESC')
            ->getQuery()
            ->getResult();
    }

    private function generateRssFeed(array $posts, string $baseUrl, string $locale, Request $request): \SimpleXMLElement
    {
        $rssFeed = new \SimpleXMLElement("<?xml version='1.0' encoding='UTF-8'?><rss xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:media=\"http://search.yahoo.com/mrss/\" xmlns:atom=\"http://www.w3.org/2005/Atom\" version='2.0'></rss>");
        $channel = $rssFeed->addChild('channel');
        $channel->addChild('title', '4BIS Innovation Blog Articles');
        $channel->addChild('link', 'https://4bis.nl/' . $locale . '/blog');

        $atomLink = $channel->addChild('link', '');
        $atomLink->addAttribute('href', $request->getUri());
        $atomLink->addAttribute('rel', 'self');
        $atomLink->addAttribute('type', 'application/rss+xml');

        $channel->addChild('language', $locale);
        $currentYear = date('Y');
        $channel->addChild('copyright', "4bis.nl © $currentYear All rights reserved");
        $channel->addChild('description', '4bis.nl, software development and hosting partner | Maastricht, NL');

        $image = $channel->addChild('image');
        $image->addChild('title', '4BIS Innovation Blog Articles');
        $image->addChild('url', 'https://cdn.4b.is/techlabs.4bis.co/cdn/4bis-nl/4bis-nl/2022-02-25/6218c42533d9b.4BIS%20Innovations_logo.png');
        $image->addChild('link', 'https://4bis.nl/' . $locale . '/');

        foreach ($posts as $post) {
            $item = $channel->addChild('item');
            $item->addChild('title', htmlspecialchars(strip_tags($post['title'])));
            $item->addChild('link', htmlspecialchars($post['absoluteUrl']));

            $atomLink = $item->addChild('atom:link', '');
            $atomLink->addAttribute('href', htmlspecialchars($post['absoluteUrl']));
            $atomLink->addAttribute('rel', 'standout');

            $item->addChild('language', $locale);
            $item->addChild('translation', $post['translation_uuid']);

            $guid = $item->addChild('guid', $post['post_uuid']);
            $guid->addAttribute('isPermaLink', 'false');

            $item->addChild('description', htmlspecialchars(strip_tags($post['description'])));
            $item->addChild('content:encoded', htmlspecialchars(strip_tags($post['content'])));

            $pubDateTime = new \DateTime($post['publishedAt']->format('Y-m-d H:i:s'), new \DateTimeZone('UTC'));
            $pubDateFormatted = $pubDateTime->format('D, d M Y H:i:s O');
            $item->addChild('pubDate', $pubDateFormatted);
        }

        return $rssFeed;
    }
}
